@@ -8,23 +8,10 @@ import pprint
 import sys
 import warnings
 
-try:
-    from jsonschema import ValidationError
-    from jsonschema import Draft4Validator as Validator
-except ImportError as e:
-    verbose_msg = """
-
-    Jupyter notebook format depends on the jsonschema package:
-
-        https://pypi.python.org/pypi/jsonschema
-
-    Please install it first.
-    """
-    raise ImportError(str(e) + verbose_msg)
-
 from ipython_genutils.importstring import import_item
+from .json_compat import get_current_validator, ValidationError
 from .reader import get_version, reads
-
+from .corpus.words import generate_corpus_id
 
 validators = {}
 
@@ -54,7 +41,7 @@ def _allow_undefined(schema):
 def get_validator(version=None, version_minor=None, relax_add_props=False):
     """Load the JSON schema into a Validator"""
     if version is None:
-        from .. import current_nbformat
+        from . import current_nbformat
         version = current_nbformat
 
     v = import_item("nbformat.v%s" % version)
@@ -62,7 +49,8 @@ def get_validator(version=None, version_minor=None, relax_add_props=False):
     if version_minor is None:
         version_minor = current_minor
 
-    version_tuple = (version, version_minor)
+    current_validator = get_current_validator()
+    version_tuple = (current_validator.name, version, version_minor)
 
     if version_tuple not in validators:
         try:
@@ -76,7 +64,7 @@ def get_validator(version=None, version_minor=None, relax_add_props=False):
             # and allow undefined cell types and outputs
             schema_json = _allow_undefined(schema_json)
 
-        validators[version_tuple] = Validator(schema_json)
+        validators[version_tuple] = current_validator(schema_json)
 
     if relax_add_props:
         try:
@@ -87,8 +75,8 @@ def get_validator(version=None, version_minor=None, relax_add_props=False):
         # this allows properties to be added for intermediate
         # representations while validating for all other kinds of errors
         schema_json = _relax_additional_properties(schema_json)
+        validators[version_tuple] = current_validator(schema_json)
 
-        validators[version_tuple] = Validator(schema_json)
     return validators[version_tuple]
 
 
@@ -258,7 +246,6 @@ def validate(nbdict=None, ref=None, version=None, version_minor=None,
     else:
         raise TypeError("validate() missing 1 required argument: 'nbdict'")
 
-
     if ref is None:
         # if ref is not specified, we have a whole notebook, so we can get the version
         nbdict_version, nbdict_version_minor = get_version(nbdict)
@@ -271,11 +258,29 @@ def validate(nbdict=None, ref=None, version=None, version_minor=None,
         if version is None:
             version, version_minor = 1, 0
 
+    if ref is None and version >= 4 and version_minor >= 5:
+        # if we support cell ids ensure default ids are provided
+        for cell in nbdict['cells']:
+            if 'id' not in cell:
+                # Generate cell ids if any are missing
+                cell['id'] = generate_corpus_id()
+
     for error in iter_validate(nbdict, ref=ref, version=version,
                                version_minor=version_minor,
                                relax_add_props=relax_add_props):
         raise error
 
+    if ref is None and version >= 4 and version_minor >= 5:
+        # if we support cell ids check for uniqueness when validating the whole notebook
+        seen_ids = set()
+        for cell in nbdict['cells']:
+            cell_id = cell['id']
+            if cell_id in seen_ids:
+                cell['id'] = generate_corpus_id()
+                # Best effort to repair if we find a duplicate id in case the ValidationError isn't blocking
+                raise ValidationError("Non-unique cell id '{}' detected. Corrected to '{}'.".format(cell_id, cell['id']))
+            seen_ids.add(cell_id)
+
 
 def iter_validate(nbdict=None, ref=None, version=None, version_minor=None,
                   relax_add_props=False, nbjson=None):
